aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
-rw-r--r--ld/ChangeLog27
-rw-r--r--ld/Makefile.in63
-rw-r--r--ld/emulparams/a29k.sh1
-rw-r--r--ld/emulparams/ebmon29k.sh1
-rw-r--r--ld/emulparams/elf32-sparc.sh1
-rw-r--r--ld/emulparams/elf64-sparc.sh1
-rw-r--r--ld/emulparams/go32.sh1
-rw-r--r--ld/emulparams/hppaosf.sh1
-rw-r--r--ld/emulparams/sa29200.sh1
-rw-r--r--ld/emulparams/z8ksim.sh1
-rw-r--r--ld/emultempl/generic.em125
-rw-r--r--ld/emultempl/gld960.em190
-rw-r--r--ld/emultempl/hppaosf.em72
-rw-r--r--ld/emultempl/lnk960.em331
-rw-r--r--ld/emultempl/m88kbcs.em126
-rwxr-xr-xld/genscripts.sh24
16 files changed, 902 insertions, 64 deletions
diff --git a/ld/ChangeLog b/ld/ChangeLog
index 4ec2af5..509b784 100644
--- a/ld/ChangeLog
+++ b/ld/ChangeLog
@@ -1,3 +1,30 @@
+Wed Jul 21 14:28:42 1993 David J. Mackenzie (djm@thepub.cygnus.com)
+
+ * Makefile.in (GENSCRIPTS): Pass the default emulation name to
+ genscripts.sh. Pass the current emulation name without ".sh" on
+ the end.
+ * genscripts.sh: Take an default emulation arg.
+ Use the emulparams file name as EMULATION_NAME.
+ Make default lib path for cross-compiling ':', not null.
+ * emulparams/*.sh: Don't set EMULATION_NAME.
+ * ldemul.c (ldemul_get_script): Take isfile arg.
+ Pass it to emulation's get_script function.
+ * ldemul.h: Adjust get_script prototypes.
+ * ldfile.c (ldfile_find_command_file): Renamed from find_a_name.
+ No longer static.
+ * ldfile.h: Declare it.
+ * ldgram.y: Accept a script on the command line again,
+ for parsing compiled-in scripts.
+ * ldmain.c (main): If ld script is a file, parse it as a -T
+ option, otherwise parse it directly.
+ * emultempl/*.em (*get_script): Return the scripts themselves if
+ this is the default emulation; otherwise return their file names.
+ * emultempl/m88kbcs.em: New file, to take m88kbcs.em #ifdef out of
+ generic.em.
+ * emulparams/m88kbcs.sh: Use it.
+
+ * ld.h (unix_relocate): Remove unused var.
+
Tue Jul 20 12:01:49 1993 Ken Raeburn (raeburn@cambridge.cygnus.com)
* Makefile.in (ALL_EMULATIONS): Delete em_i386linux.o (for which
diff --git a/ld/Makefile.in b/ld/Makefile.in
index 5253e45..c47fc47 100644
--- a/ld/Makefile.in
+++ b/ld/Makefile.in
@@ -180,6 +180,7 @@ ldgram.h ldgram.c: ldgram.y
mv -f y.tab.c ldgram.c
mv -f y.tab.h ldgram.h
+# EMUL is the name of a file in the emulparams subdir, without the .sh.
DEF_EMUL = ` if [ -z "$(EMUL)" ] ; then \
echo "you must set a default emulation" 1>&2 ; \
exit 1 ; \
@@ -212,99 +213,99 @@ ldlex.c: ldlex.l ldgram.h
# These all start with em_ so 'make clean' can find them.
-GENSCRIPTS=sh $(srcdir)/genscripts.sh ${srcdir} ${libdir} ${host_alias} ${target_alias}
+GENSCRIPTS=sh $(srcdir)/genscripts.sh ${srcdir} ${libdir} ${host_alias} ${target_alias} ${EMUL}
GEN_DEPENDS=$(srcdir)/genscripts.sh
em_sun4.c: $(srcdir)/emulparams/sun4.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} sun4.sh
+ ${GENSCRIPTS} sun4
em_sun3.c: $(srcdir)/emulparams/sun3.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} sun3.sh
+ ${GENSCRIPTS} sun3
em_go32.c: $(srcdir)/emulparams/go32.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} go32.sh
+ ${GENSCRIPTS} go32
em_news.c: $(srcdir)/emulparams/news.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} news.sh
+ ${GENSCRIPTS} news
em_vax.c: $(srcdir)/emulparams/vax.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} vax.sh
+ ${GENSCRIPTS} vax
em_hp300bsd.c: $(srcdir)/emulparams/hp300bsd.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} hp300bsd.sh
+ ${GENSCRIPTS} hp300bsd
em_hp3hpux.c: $(srcdir)/emulparams/hp3hpux.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} hp3hpux.sh
+ ${GENSCRIPTS} hp3hpux
em_hppaosf.c: $(srcdir)/emulparams/hppaosf.sh \
$(srcdir)/emultempl/hppaosf.em $(srcdir)/scripttempl/hppaosf.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} hppaosf.sh
+ ${GENSCRIPTS} hppaosf
em_i386aout.c: $(srcdir)/emulparams/i386aout.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} i386aout.sh
+ ${GENSCRIPTS} i386aout
em_ebmon29k.c: $(srcdir)/emulparams/ebmon29k.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/ebmon29k.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} ebmon29k.sh
+ ${GENSCRIPTS} ebmon29k
em_sa29200.c: $(srcdir)/emulparams/sa29200.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/sa29200.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} sa29200.sh
+ ${GENSCRIPTS} sa29200
em_a29k.c: $(srcdir)/emulparams/a29k.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/a29k.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} a29k.sh
+ ${GENSCRIPTS} a29k
em_m88kbcs.c: $(srcdir)/emulparams/m88kbcs.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/m88kbcs.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} m88kbcs.sh
+ ${GENSCRIPTS} m88kbcs
em_h8300.c: $(srcdir)/emulparams/h8300.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/h8300.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} h8300.sh
+ ${GENSCRIPTS} h8300
em_h8300h.c: $(srcdir)/emulparams/h8300h.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/h8300h.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} h8300h.sh
+ ${GENSCRIPTS} h8300h
em_h8500.c: $(srcdir)/emulparams/h8500.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/h8500.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} h8500.sh
+ ${GENSCRIPTS} h8500
em_sh.c: $(srcdir)/emulparams/sh.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/sh.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} sh.sh
+ ${GENSCRIPTS} sh
em_st2000.c: $(srcdir)/emulparams/st2000.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/st2000.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} st2000.sh
+ ${GENSCRIPTS} st2000
em_z8ksim.c: $(srcdir)/emulparams/z8ksim.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/z8ksim.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} z8ksim.sh
+ ${GENSCRIPTS} z8ksim
em_vanilla.c: $(srcdir)/emulparams/vanilla.sh \
$(srcdir)/emultempl/vanilla.em $(srcdir)/scripttempl/vanilla.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} vanilla.sh
+ ${GENSCRIPTS} vanilla
em_lnk960.c: $(srcdir)/emulparams/lnk960.sh \
$(srcdir)/emultempl/lnk960.em $(srcdir)/scripttempl/i960.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} lnk960.sh
+ ${GENSCRIPTS} lnk960
em_gld960.c: $(srcdir)/emulparams/gld960.sh \
$(srcdir)/emultempl/gld960.em $(srcdir)/scripttempl/i960.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} gld960.sh
+ ${GENSCRIPTS} gld960
em_m68kcoff.c: $(srcdir)/emulparams/m68kcoff.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/m68kcoff.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} m68kcoff.sh
+ ${GENSCRIPTS} m68kcoff
em_i386coff.c: $(srcdir)/emulparams/i386coff.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/i386coff.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} i386coff.sh
+ ${GENSCRIPTS} i386coff
em_mipslit.c: $(srcdir)/emulparams/mipslit.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/mips.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} mipslit.sh
+ ${GENSCRIPTS} mipslit
em_i386bsd.c: $(srcdir)/emulparams/i386bsd.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} i386bsd.sh
+ ${GENSCRIPTS} i386bsd
em_mipsbig.c: $(srcdir)/emulparams/mipsbig.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/mips.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} mipsbig.sh
+ ${GENSCRIPTS} mipsbig
em_mipsbsd.c: $(srcdir)/emulparams/mipsbsd.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/aout.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} mipsbsd.sh
+ ${GENSCRIPTS} mipsbsd
em_mipsidt.c: $(srcdir)/emulparams/mipsidt.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/mips.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} mipsidt.sh
+ ${GENSCRIPTS} mipsidt
em_mipsidtl.c: $(srcdir)/emulparams/mipsidtl.sh \
$(srcdir)/emultempl/generic.em $(srcdir)/scripttempl/mips.sc ${GEN_DEPENDS}
- ${GENSCRIPTS} mipsidtl.sh
+ ${GENSCRIPTS} mipsidtl
$(LD_PROG): $(OFILES) $(BFDLIB) $(LIBIBERTY)
$(CC) $(INCLUDES) $(HDEFINES) $(TDEFINES) $(CDEFINES) $(LDFLAGS) -o $(LD_PROG) $(OFILES) $(BFDLIB) $(LIBIBERTY) $(LOADLIBES) $(CFLAGS)
diff --git a/ld/emulparams/a29k.sh b/ld/emulparams/a29k.sh
index f1b7dea..43a85e8 100644
--- a/ld/emulparams/a29k.sh
+++ b/ld/emulparams/a29k.sh
@@ -1,4 +1,3 @@
-EMULATION_NAME=a29k
SCRIPT_NAME=a29k
OUTPUT_FORMAT="coff-a29k-big"
TEXT_START_ADDR=0x1000000
diff --git a/ld/emulparams/ebmon29k.sh b/ld/emulparams/ebmon29k.sh
index a7c3634..43ef5bd 100644
--- a/ld/emulparams/ebmon29k.sh
+++ b/ld/emulparams/ebmon29k.sh
@@ -1,4 +1,3 @@
-EMULATION_NAME=ebmon29k
SCRIPT_NAME=ebmon29k
OUTPUT_FORMAT="coff-a29k-big"
TEXT_START_ADDR=0x8000
diff --git a/ld/emulparams/elf32-sparc.sh b/ld/emulparams/elf32-sparc.sh
index 6e647b8..93a0b4e 100644
--- a/ld/emulparams/elf32-sparc.sh
+++ b/ld/emulparams/elf32-sparc.sh
@@ -1,4 +1,3 @@
-EMULATION_NAME=elf32_sparc
SCRIPT_NAME=elf
OUTPUT_FORMAT="elf32-sparc"
TEXT_START_ADDR=0x100200
diff --git a/ld/emulparams/elf64-sparc.sh b/ld/emulparams/elf64-sparc.sh
index 6f74222..985de05 100644
--- a/ld/emulparams/elf64-sparc.sh
+++ b/ld/emulparams/elf64-sparc.sh
@@ -1,4 +1,3 @@
-EMULATION_NAME=elf64_sparc
SCRIPT_NAME=elf
OUTPUT_FORMAT="elf64-sparc"
TEXT_START_ADDR=0x100200
diff --git a/ld/emulparams/go32.sh b/ld/emulparams/go32.sh
index cd548d8..5b8f7dc 100644
--- a/ld/emulparams/go32.sh
+++ b/ld/emulparams/go32.sh
@@ -1,4 +1,3 @@
-EMULATION_NAME=go32
SCRIPT_NAME=aout
OUTPUT_FORMAT="a.out-i386"
TEXT_START_ADDR=0x1020
diff --git a/ld/emulparams/hppaosf.sh b/ld/emulparams/hppaosf.sh
index ba34bf4..c51582c 100644
--- a/ld/emulparams/hppaosf.sh
+++ b/ld/emulparams/hppaosf.sh
@@ -1,4 +1,3 @@
-EMULATION_NAME=hppaosf
SCRIPT_NAME=hppaosf
OUTPUT_FORMAT="elf-big"
TEXT_START_ADDR=0x1000
diff --git a/ld/emulparams/sa29200.sh b/ld/emulparams/sa29200.sh
index 96b2846..91c07d3 100644
--- a/ld/emulparams/sa29200.sh
+++ b/ld/emulparams/sa29200.sh
@@ -1,4 +1,3 @@
-EMULATION_NAME=sa29200
SCRIPT_NAME=sa29200
OUTPUT_FORMAT="coff-a29k-big"
TEXT_START_ADDR=0x40004000
diff --git a/ld/emulparams/z8ksim.sh b/ld/emulparams/z8ksim.sh
index 1fa1eb3..aee5957 100644
--- a/ld/emulparams/z8ksim.sh
+++ b/ld/emulparams/z8ksim.sh
@@ -1,4 +1,3 @@
-EMULATION_NAME=z8ksim
SCRIPT_NAME=z8ksim
OUTPUT_FORMAT="coff-z8k"
TEXT_START_ADDR=0x0
diff --git a/ld/emultempl/generic.em b/ld/emultempl/generic.em
new file mode 100644
index 0000000..43af826
--- /dev/null
+++ b/ld/emultempl/generic.em
@@ -0,0 +1,125 @@
+# This shell script emits a C file. -*- C -*-
+# It does some substitutions.
+cat >em_${EMULATION_NAME}.c <<EOF
+/* This file is is generated by a shell script. DO NOT EDIT! */
+
+/* emulate the original gld for the given ${EMULATION_NAME}
+ Copyright (C) 1991 Free Software Foundation, Inc.
+ Written by Steve Chamberlain steve@cygnus.com
+
+This file is part of GLD, the Gnu Linker.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
+
+#define TARGET_IS_${EMULATION_NAME}
+
+#include "bfd.h"
+#include "sysdep.h"
+#include "ld.h"
+#include "config.h"
+#include "ldemul.h"
+#include "ldfile.h"
+#include "ldmisc.h"
+
+extern boolean lang_float_flag;
+
+extern enum bfd_architecture ldfile_output_architecture;
+extern unsigned long ldfile_output_machine;
+extern char *ldfile_output_machine_name;
+
+static void
+gld${EMULATION_NAME}_before_parse()
+{
+#ifndef TARGET_ /* I.e., if not generic. */
+ ldfile_output_architecture = bfd_arch_${ARCH};
+#endif /* not TARGET_ */
+}
+
+static char *
+gld${EMULATION_NAME}_get_script(isfile)
+ int *isfile;
+EOF
+
+if test "$DEFAULT_EMULATION" = "$EMULATION_NAME"
+then
+# Scripts compiled in.
+
+# sed commands to quote an ld script as a C string.
+sc='s/["\\]/\\&/g
+s/$/\\n\\/
+1s/^/"{/
+$s/$/n}"/
+'
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 0;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xu`;
+ else if (config.relocateable_output == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xr`;
+ else if (!config.text_read_only)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xbn`;
+ else if (!config.magic_demand_paged)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xn`;
+ else
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.x`;
+}
+EOF
+
+else
+# Scripts read from the filesystem.
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 1;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return "ldscripts/${EMULATION_NAME}.xu";
+ else if (config.relocateable_output == true)
+ return "ldscripts/${EMULATION_NAME}.xr";
+ else if (!config.text_read_only)
+ return "ldscripts/${EMULATION_NAME}.xbn";
+ else if (!config.magic_demand_paged)
+ return "ldscripts/${EMULATION_NAME}.xn";
+ else
+ return "ldscripts/${EMULATION_NAME}.x";
+}
+EOF
+
+fi
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+
+struct ld_emulation_xfer_struct ld_${EMULATION_NAME}_emulation =
+{
+ gld${EMULATION_NAME}_before_parse,
+ syslib_default,
+ hll_default,
+ after_parse_default,
+ after_allocation_default,
+ set_output_arch_default,
+ ldemul_default_target,
+ before_allocation_default,
+ gld${EMULATION_NAME}_get_script,
+ "${EMULATION_NAME}",
+ "${OUTPUT_FORMAT}"
+};
+EOF
diff --git a/ld/emultempl/gld960.em b/ld/emultempl/gld960.em
new file mode 100644
index 0000000..69555e2
--- /dev/null
+++ b/ld/emultempl/gld960.em
@@ -0,0 +1,190 @@
+# This shell script emits a C file. -*- C -*-
+# It does some substitutions.
+cat >em_${EMULATION_NAME}.c <<EOF
+/* Copyright (C) 1991 Free Software Foundation, Inc.
+
+This file is part of GLD, the Gnu Linker.
+
+GLD is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
+GLD is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GLD; see the file COPYING. If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+
+/*
+ * emulate the Intels port of gld
+ */
+
+
+#include "bfd.h"
+#include "sysdep.h"
+
+
+#include "ld.h"
+#include "config.h"
+#include "ldemul.h"
+#include "ldfile.h"
+#include "ldmisc.h"
+
+
+/* IMPORTS */
+extern char *output_filename;
+extern boolean lang_float_flag;
+
+
+extern enum bfd_architecture ldfile_output_architecture;
+extern unsigned long ldfile_output_machine;
+extern char *ldfile_output_machine_name;
+
+extern bfd *output_bfd;
+
+
+
+#ifdef GNU960
+
+static void
+gld960_before_parse()
+{
+ static char *env_variables[] = { "G960LIB", "G960BASE", 0 };
+ char **p;
+ char *env ;
+
+ for ( p = env_variables; *p; p++ ){
+ env = (char *) getenv(*p);
+ if (env) {
+ ldfile_add_library_path(concat(env,"/lib/libbout",""));
+ }
+ }
+ ldfile_output_architecture = bfd_arch_i960;
+}
+
+#else /* not GNU960 */
+
+static void gld960_before_parse()
+{
+ char *env ;
+ env = getenv("G960LIB");
+ if (env) {
+ ldfile_add_library_path(env);
+ }
+ env = getenv("G960BASE");
+ if (env) {
+ ldfile_add_library_path(concat(env,"/lib",""));
+ }
+ ldfile_output_architecture = bfd_arch_i960;
+}
+
+#endif /* GNU960 */
+
+
+static void
+gld960_set_output_arch()
+{
+ bfd_set_arch_mach(output_bfd, ldfile_output_architecture, bfd_mach_i960_core);
+}
+
+static char *
+gld960_choose_target()
+{
+#ifdef GNU960
+
+ output_filename = "b.out";
+ return bfd_make_targ_name(BFD_BOUT_FORMAT, 0);
+
+#else
+
+ char *from_outside = getenv(TARGET_ENVIRON);
+ output_filename = "b.out";
+
+ if (from_outside != (char *)NULL)
+ return from_outside;
+
+ return "b.out.little";
+
+#endif
+}
+
+static char *
+gld960_get_script(isfile)
+ int *isfile;
+EOF
+
+if test "$DEFAULT_EMULATION" = "$EMULATION_NAME"
+then
+# Scripts compiled in.
+
+# sed commands to quote an ld script as a C string.
+sc='s/["\\]/\\&/g
+s/$/\\n\\/
+1s/^/"{/
+$s/$/n}"/
+'
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 0;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xu`;
+ else if (config.relocateable_output == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xr`;
+ else if (!config.text_read_only)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xbn`;
+ else if (!config.magic_demand_paged)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xn`;
+ else
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.x`;
+}
+EOF
+
+else
+# Scripts read from the filesystem.
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 1;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return "ldscripts/${EMULATION_NAME}.xu";
+ else if (config.relocateable_output == true)
+ return "ldscripts/${EMULATION_NAME}.xr";
+ else if (!config.text_read_only)
+ return "ldscripts/${EMULATION_NAME}.xbn";
+ else if (!config.magic_demand_paged)
+ return "ldscripts/${EMULATION_NAME}.xn";
+ else
+ return "ldscripts/${EMULATION_NAME}.x";
+}
+EOF
+
+fi
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+
+struct ld_emulation_xfer_struct ld_gld960_emulation =
+{
+ gld960_before_parse,
+ syslib_default,
+ hll_default,
+ after_parse_default,
+ after_allocation_default,
+ gld960_set_output_arch,
+ gld960_choose_target,
+ before_allocation_default,
+ gld960_get_script,
+ "960",
+ ""
+};
+EOF
diff --git a/ld/emultempl/hppaosf.em b/ld/emultempl/hppaosf.em
index b066f6b..d08951d 100644
--- a/ld/emultempl/hppaosf.em
+++ b/ld/emultempl/hppaosf.em
@@ -1,3 +1,5 @@
+# This shell script emits a C file. -*- C -*-
+# It does some substitutions.
cat >em_${EMULATION_NAME}.c <<EOF
/* An emulation for HP PA-RISC OSF/1 linkers.
Copyright (C) 1991 Free Software Foundation, Inc.
@@ -79,24 +81,66 @@ hppaosf_set_output_arch()
bfd_set_arch_mach(output_bfd, ldfile_output_architecture, machine);
}
-static char *script =
-"hppaosf.x"
-;
+static char *
+hppaosf_get_script(isfile)
+ int *isfile;
+EOF
-
-static char *script_reloc =
-"hppaosf.xr"
- ;
+if test "$DEFAULT_EMULATION" = "$EMULATION_NAME"
+then
+# Scripts compiled in.
+
+# sed commands to quote an ld script as a C string.
+sc='s/["\\]/\\&/g
+s/$/\\n\\/
+1s/^/"{/
+$s/$/n}"/
+'
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 0;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xu`;
+ else if (config.relocateable_output == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xr`;
+ else if (!config.text_read_only)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xbn`;
+ else if (!config.magic_demand_paged)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xn`;
+ else
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.x`;
+}
+EOF
+else
+# Scripts read from the filesystem.
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 1;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return "ldscripts/${EMULATION_NAME}.xu";
+ else if (config.relocateable_output == true)
+ return "ldscripts/${EMULATION_NAME}.xr";
+ else if (!config.text_read_only)
+ return "ldscripts/${EMULATION_NAME}.xbn";
+ else if (!config.magic_demand_paged)
+ return "ldscripts/${EMULATION_NAME}.xn";
+ else
+ return "ldscripts/${EMULATION_NAME}.x";
+}
+EOF
-static char *hppaosf_get_script()
-{
- extern ld_config_type config;
- if (config.relocateable_output)
- return script_reloc;
- return script;
+fi
-}
+cat >>em_${EMULATION_NAME}.c <<EOF
struct ld_emulation_xfer_struct ld_hppaosf_emulation =
{
diff --git a/ld/emultempl/lnk960.em b/ld/emultempl/lnk960.em
new file mode 100644
index 0000000..4a4cc07
--- /dev/null
+++ b/ld/emultempl/lnk960.em
@@ -0,0 +1,331 @@
+# This shell script emits a C file. -*- C -*-
+# It does some substitutions.
+cat >em_${EMULATION_NAME}.c <<EOF
+/* intel coff loader emulation specific stuff
+ Copyright (C) 1991 Free Software Foundation, Inc.
+ Written by Steve Chamberlain steve@cygnus.com
+
+This file is part of GLD, the Gnu Linker.
+
+GLD is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+GLD is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GLD; see the file COPYING. If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+
+#include "bfd.h"
+#include "sysdep.h"
+
+/*#include "archures.h"*/
+#include "ld.h"
+#include "config.h"
+#include "ldemul.h"
+#include "ldmisc.h"
+#include "ldlang.h"
+#include "ldfile.h"
+
+extern boolean lang_float_flag;
+extern bfd *output_bfd;
+
+
+
+extern enum bfd_architecture ldfile_output_architecture;
+extern unsigned long ldfile_output_machine;
+extern char *ldfile_output_machine_name;
+
+
+typedef struct lib_list {
+ char *name;
+ struct lib_list *next;
+} lib_list_type;
+
+static lib_list_type *hll_list;
+static lib_list_type **hll_list_tail = &hll_list;
+
+static lib_list_type *syslib_list;
+static lib_list_type **syslib_list_tail = &syslib_list;
+
+
+static void
+append(list, name)
+lib_list_type ***list;
+char *name;
+{
+ lib_list_type *element =
+ (lib_list_type *)(ldmalloc(sizeof(lib_list_type)));
+
+ element->name = name;
+ element->next = (lib_list_type *)NULL;
+ **list = element;
+ *list = &element->next;
+
+}
+
+static boolean had_hll = false;
+static boolean had_hll_name = false;
+static void
+lnk960_hll(name)
+char *name;
+{
+ had_hll = true;
+ if (name != (char *)NULL) {
+ had_hll_name = true;
+ append(&hll_list_tail, name);
+ }
+}
+
+static void
+lnk960_syslib(name)
+char *name;
+{
+ append(&syslib_list_tail,name);
+}
+
+
+
+#ifdef GNU960
+
+static void
+lnk960_before_parse()
+{
+ static char *env_variables[] = { "G960LIB", "G960BASE", 0 };
+ char **p;
+ char *env ;
+
+ for ( p = env_variables; *p; p++ ){
+ env = (char *) getenv(*p);
+ if (env) {
+ ldfile_add_library_path(concat(env,"/lib/libcoff",""));
+ }
+ }
+
+ env= (char *) getenv("I960BASE");
+ if ( env ) {
+ ldfile_add_library_path(concat(env,"/lib",""));
+ }
+
+ ldfile_output_architecture = bfd_arch_i960;
+ ldfile_output_machine = bfd_mach_i960_core;
+}
+
+#else /* not GNU960 */
+
+static void
+lnk960_before_parse()
+{
+ char *name = getenv("I960BASE");
+
+ if (name == (char *)NULL) {
+ name = getenv("G960BASE");
+ if (name == (char *)NULL) {
+ einfo("%P%F I960BASE and G960BASE not set\n");
+ }
+ }
+
+
+ ldfile_add_library_path(concat(name,"/lib",""));
+ ldfile_output_architecture = bfd_arch_i960;
+ ldfile_output_machine = bfd_mach_i960_core;
+}
+
+#endif /* GNU960 */
+
+
+static void
+add_on(list, search)
+lib_list_type *list;
+lang_input_file_enum_type search;
+{
+ while (list) {
+ lang_add_input_file(list->name,
+ search,
+ (char *)NULL);
+ list = list->next;
+ }
+}
+static void lnk960_after_parse()
+{
+
+ /* If there has been no arch, default to -KB */
+ if (ldfile_output_machine_name[0] ==0) {
+ ldfile_add_arch("KB");
+ }
+
+ /* if there has been no hll list then add our own */
+
+ if(had_hll && !had_hll_name) {
+ append(&hll_list_tail,"cg");
+ if (ldfile_output_machine == bfd_mach_i960_ka_sa ||
+ ldfile_output_machine == bfd_mach_i960_ca) {
+ {
+ append(&hll_list_tail,"fpg");
+ }
+ }
+ }
+
+ add_on(hll_list, lang_input_file_is_l_enum);
+ add_on(syslib_list, lang_input_file_is_search_file_enum);
+}
+
+static void
+lnk960_before_allocation()
+{
+}
+static void
+lnk960_after_allocation()
+{
+ extern ld_config_type config;
+ if (config.relocateable_output == false) {
+ lang_abs_symbol_at_end_of(".text","_etext");
+ lang_abs_symbol_at_end_of(".data","_edata");
+ lang_abs_symbol_at_beginning_of(".bss","_bss_start");
+ lang_abs_symbol_at_end_of(".bss","_end");
+ }
+}
+
+
+static struct
+ {
+ unsigned long number;
+ char *name;
+ }
+machine_table[] = {
+ bfd_mach_i960_core ,"CORE",
+ bfd_mach_i960_kb_sb ,"KB",
+ bfd_mach_i960_kb_sb ,"SB",
+ bfd_mach_i960_mc ,"MC",
+ bfd_mach_i960_xa ,"XA",
+ bfd_mach_i960_ca ,"CA",
+ bfd_mach_i960_ka_sa ,"KA",
+ bfd_mach_i960_ka_sa ,"SA",
+
+ bfd_mach_i960_core ,"core",
+ bfd_mach_i960_kb_sb ,"kb",
+ bfd_mach_i960_kb_sb ,"sb",
+ bfd_mach_i960_mc ,"mc",
+ bfd_mach_i960_xa ,"xa",
+ bfd_mach_i960_ca ,"ca",
+ bfd_mach_i960_ka_sa ,"ka",
+ bfd_mach_i960_ka_sa ,"sa",
+ 0,(char *)NULL
+};
+
+static void
+lnk960_set_output_arch()
+{
+ /* Set the output architecture and machine if possible */
+ unsigned int i;
+ ldfile_output_machine = bfd_mach_i960_core;
+ for (i= 0; machine_table[i].name != (char*)NULL; i++) {
+ if (strcmp(ldfile_output_machine_name,machine_table[i].name)==0) {
+ ldfile_output_machine = machine_table[i].number;
+ break;
+ }
+ }
+ bfd_set_arch_mach(output_bfd, ldfile_output_architecture, ldfile_output_machine);
+}
+
+static char *
+lnk960_choose_target()
+{
+#ifdef GNU960
+
+ return bfd_make_targ_name(BFD_COFF_FORMAT, 0);
+
+#else
+
+ char *from_outside = getenv(TARGET_ENVIRON);
+ if (from_outside != (char *)NULL)
+ return from_outside;
+#ifdef LNK960_LITTLE
+ return "coff-Intel-little";
+#else
+ return "coff-Intel-big";
+#endif
+#endif
+
+}
+
+static char *
+lnk960_get_script(isfile)
+ int *isfile;
+EOF
+
+if test "$DEFAULT_EMULATION" = "$EMULATION_NAME"
+then
+# Scripts compiled in.
+
+# sed commands to quote an ld script as a C string.
+sc='s/["\\]/\\&/g
+s/$/\\n\\/
+1s/^/"{/
+$s/$/n}"/
+'
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+ extern ld_config_type config;
+
+ *isfile = 0;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xu`;
+ else if (config.relocateable_output == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xr`;
+ else if (!config.text_read_only)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xbn`;
+ else if (!config.magic_demand_paged)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xn`;
+ else
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.x`;
+}
+EOF
+
+else
+# Scripts read from the filesystem.
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 1;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return "ldscripts/${EMULATION_NAME}.xu";
+ else if (config.relocateable_output == true)
+ return "ldscripts/${EMULATION_NAME}.xr";
+ else if (!config.text_read_only)
+ return "ldscripts/${EMULATION_NAME}.xbn";
+ else if (!config.magic_demand_paged)
+ return "ldscripts/${EMULATION_NAME}.xn";
+ else
+ return "ldscripts/${EMULATION_NAME}.x";
+}
+EOF
+
+fi
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+
+struct ld_emulation_xfer_struct ld_lnk960_emulation =
+{
+ lnk960_before_parse,
+ lnk960_syslib,
+ lnk960_hll,
+ lnk960_after_parse,
+ lnk960_after_allocation,
+ lnk960_set_output_arch,
+ lnk960_choose_target,
+ lnk960_before_allocation,
+ lnk960_get_script,
+ "lnk960",
+ ""
+};
+EOF
diff --git a/ld/emultempl/m88kbcs.em b/ld/emultempl/m88kbcs.em
new file mode 100644
index 0000000..52682e6
--- /dev/null
+++ b/ld/emultempl/m88kbcs.em
@@ -0,0 +1,126 @@
+# This shell script emits a C file. -*- C -*-
+# It does some substitutions.
+cat >em_${EMULATION_NAME}.c <<EOF
+/* This file is is generated by a shell script. DO NOT EDIT! */
+
+/* emulate the original gld for the given ${EMULATION_NAME}
+ Copyright (C) 1991 Free Software Foundation, Inc.
+ Written by Steve Chamberlain steve@cygnus.com
+
+This file is part of GLD, the Gnu Linker.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
+
+#define TARGET_IS_${EMULATION_NAME}
+
+#include "bfd.h"
+#include "sysdep.h"
+#include "ld.h"
+#include "config.h"
+#include "ldemul.h"
+#include "ldfile.h"
+#include "ldmisc.h"
+
+extern boolean lang_float_flag;
+
+extern enum bfd_architecture ldfile_output_architecture;
+extern unsigned long ldfile_output_machine;
+extern char *ldfile_output_machine_name;
+
+static void
+gld${EMULATION_NAME}_before_parse()
+{
+ extern char lprefix;
+ lprefix = '@';
+
+ ldfile_output_architecture = bfd_arch_${ARCH};
+}
+
+static char *
+gld${EMULATION_NAME}_get_script(isfile)
+ int *isfile;
+EOF
+
+if test "$DEFAULT_EMULATION" = "$EMULATION_NAME"
+then
+# Scripts compiled in.
+
+# sed commands to quote an ld script as a C string.
+sc='s/["\\]/\\&/g
+s/$/\\n\\/
+1s/^/"{/
+$s/$/n}"/
+'
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 0;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xu`;
+ else if (config.relocateable_output == true)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xr`;
+ else if (!config.text_read_only)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xbn`;
+ else if (!config.magic_demand_paged)
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.xn`;
+ else
+ return `sed "$sc" ldscripts/${EMULATION_NAME}.x`;
+}
+EOF
+
+else
+# Scripts read from the filesystem.
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+{
+ extern ld_config_type config;
+
+ *isfile = 1;
+
+ if (config.relocateable_output == true && config.build_constructors == true)
+ return "ldscripts/${EMULATION_NAME}.xu";
+ else if (config.relocateable_output == true)
+ return "ldscripts/${EMULATION_NAME}.xr";
+ else if (!config.text_read_only)
+ return "ldscripts/${EMULATION_NAME}.xbn";
+ else if (!config.magic_demand_paged)
+ return "ldscripts/${EMULATION_NAME}.xn";
+ else
+ return "ldscripts/${EMULATION_NAME}.x";
+}
+EOF
+
+fi
+
+cat >>em_${EMULATION_NAME}.c <<EOF
+
+struct ld_emulation_xfer_struct ld_${EMULATION_NAME}_emulation =
+{
+ gld${EMULATION_NAME}_before_parse,
+ syslib_default,
+ hll_default,
+ after_parse_default,
+ after_allocation_default,
+ set_output_arch_default,
+ ldemul_default_target,
+ before_allocation_default,
+ gld${EMULATION_NAME}_get_script,
+ "${EMULATION_NAME}",
+ "${OUTPUT_FORMAT}"
+};
+EOF
diff --git a/ld/genscripts.sh b/ld/genscripts.sh
index a6a0e2e..dd1e9dd 100755
--- a/ld/genscripts.sh
+++ b/ld/genscripts.sh
@@ -1,18 +1,23 @@
# genscripts.sh
# This shell script does the work of generating the ld-emulation-target
# specific information from a specific file of paramaters.
# Usage: genscripts.sh srcdir libdir host_alias target_alias emulation_name
# Sample usage:
-# genscripts.sh /offsite/djm/work/devo/ld /usr/local/lib sparc-sun-sunos4.1.3 sparc-sun-sunos4.1.3 sun3.sh
+# genscripts.sh /djm/ld-devo/devo/ld /usr/local/lib sparc-sun-sunos4.1.3 \
+# sparc-sun-sunos4.1.3 sun4 sun3
# produces sun3.x sun3.xbn sun3.xn sun3.xr sun3.xu em_sun3.c
srcdir=$1
libdir=$2
host_alias=$3
target_alias=$4
+DEFAULT_EMULATION=$5
+EMULATION_NAME=$6
# Include the emulation-specific parameters:
-. ${srcdir}/emulparams/$5
+. ${srcdir}/emulparams/${EMULATION_NAME}.sh
# Set the library search path, for libraries named by -lfoo.
# If LIB_PATH is defined (e.g., by Makefile) and non-empty, it is used.
@@ -30,7 +35,7 @@ if [ "x${LIB_PATH}" = "x" ] ; then
fi
else
# Cross.
- LIB_PATH=
+ LIB_PATH=:
fi
fi
LIB_SEARCH_DIRS=`echo ${LIB_PATH} | tr ':' ' ' | sed -e 's/\([^ ][^ ]*\)/SEARCH_DIR(\1);/g'`
@@ -38,6 +43,7 @@ LIB_SEARCH_DIRS=`echo ${LIB_PATH} | tr ':' ' ' | sed -e 's/\([^ ][^ ]*\)/SEARCH_
# Generate 5 script files from a master script template in
# ${srcdir}/scripttempl/${SCRIPT_NAME}.sh. Which one of the 5 script files
# is actually used depends on command line options given to ld.
+# (SCRIPT_NAME was set in the emulparams_file.)
#
# A .x script file is the default script.
# A .xr script is for linking without relocation (-r flag).
@@ -85,9 +91,5 @@ DATA_ALIGNMENT=${DATA_ALIGNMENT_N}
(. ${srcdir}/scripttempl/${SCRIPT_NAME}.sc) | sed -e '/^ *$/d' > \
ldscripts/${EMULATION_NAME}.xbn
-#sed -e s/"<ldtarget>"/${EMULATION_NAME}/g -e s/"<arch>"/${ARCH}/g \
-# -e s/"<target>"/${EMULATION_NAME}/g -e s/"<target_name>"/${OUTPUT_FORMAT}/g \
-# <${srcdir}/${TEMPLATE_NAME-ldtemplate} >em_${EMULATION_NAME}.c
-
# Generate em_${EMULATION_NAME}.c.
. ${srcdir}/emultempl/${TEMPLATE_NAME-generic}.em
@@ -1,18 +1,23 @@
-# genscripts.sh
-# This shell script does the work of generating the ld-emulation-target
-# specific information from a specific file of paramaters.
-# Usage: genscripts.sh srcdir libdir host_alias target_alias emulation_name
+#!/bin/sh
+# genscripts.sh - generate the ld-emulation-target specific files
+#
+# Usage: genscripts.sh srcdir libdir host_alias target_alias \
+# default_emulation this_emulation
+#
# Sample usage: